@@ -146,6 +146,7 @@ impl TypedOp for TypedConcat {
         model: &TypedModel,
         node: &TypedNode,
         patch: &mut TypedModelPatch,
+        suffix: &str,
         _output_slot: usize,
         axis: usize,
         start: usize,
@@ -174,6 +175,7 @@ impl TypedOp for TypedConcat {
                                 model,
                                 &prec,
                                 patch,
+                                suffix,
                                 node.inputs[input].slot,
                                 axis,
                                 start - offsets[ix],
