@@ -459,6 +459,36 @@ fn group_11() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn group_12() -> anyhow::Result<()> {
+    let pb = ConvProblem {
+        shape_in: DataFormat::HWC.from_n_c_hw(1, 2, &[1])?,
+        shape_out: DataFormat::HWC.from_n_c_hw(1, 2, &[1])?,
+        kernel_format: KernelFormat::HWIO,
+        group: 2,
+        data: tract_ndarray::arr2(&[[0.0, 0.0]]).into_dyn(),
+        kernel: tract_ndarray::arr3(&[[[0.0], [0.0]]]).into_dyn(),
+        bias: None,
+    };
+    assert_eq!(pb.tract().unwrap(), pb.reference());
+    Ok(())
+}
+
+#[test]
+fn group_13() -> anyhow::Result<()> {
+    let pb = ConvProblem {
+        shape_in: DataFormat::HWC.from_n_c_hw(1, 2, &[1])?,
+        shape_out: DataFormat::HWC.from_n_c_hw(1, 4, &[1])?,
+        kernel_format: KernelFormat::HWIO,
+        group: 2,
+        data: tract_ndarray::arr2(&[[0.0, 1.0]]).into_dyn(),
+        kernel: tract_ndarray::arr3(&[[[0.0, 0.0], [1.0, 0.0]]]).into_dyn(),
+        bias: None,
+    };
+    assert_eq!(pb.tract().unwrap(), pb.reference());
+    Ok(())
+}
+
 #[test]
 fn group_bias_0() -> anyhow::Result<()> {
     let pb = ConvProblem {
@@ -520,7 +550,7 @@ fn batch_0() -> anyhow::Result<()> {
 }
 
 #[test]
-fn bias_3d() -> anyhow::Result<()> {
+fn bias_3d_1() -> anyhow::Result<()> {
     let pb = ConvProblem {
         shape_in: DataFormat::CHW.from_n_c_hw(1, 1, &[1, 1, 2])?,
         shape_out: DataFormat::CHW.from_n_c_hw(1, 1, &[1, 1, 2])?,
@@ -533,3 +563,18 @@ fn bias_3d() -> anyhow::Result<()> {
     assert_eq!(pb.tract().unwrap(), pb.reference());
     Ok(())
 }
+
+#[test]
+fn batch_3d() -> anyhow::Result<()> {
+    let pb = ConvProblem {
+        shape_in: DataFormat::NCHW.from_n_c_hw(1, 1, &[2, 2, 1])?,
+        shape_out: DataFormat::NCHW.from_n_c_hw(1, 1, &[2, 2, 1])?,
+        kernel_format: KernelFormat::OIHW,
+        group: 1,
+        data: ndarray::ArrayD::<f32>::zeros(vec![1, 1, 2, 2, 1]),
+        kernel: ndarray::ArrayD::<f32>::zeros(vec![1, 1, 1, 1, 1]),
+        bias: None,
+    };
+    assert_eq!(pb.tract().unwrap(), pb.reference());
+    Ok(())
+}
